// Copyright 2023 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:build goexperiment.systemcrypto

package ed25519

import (
	boring "crypto/internal/backend"
	"crypto/internal/boring/bcache"
	"unsafe"
)

var pubCache bcache.Cache[byte, boringPub]
var privCache bcache.Cache[byte, boringPriv]

func init() {
	if boring.Enabled && boring.SupportsEd25519() {
		pubCache.Register()
		privCache.Register()
	}
}

type boringPub struct {
	key  *boring.PublicKeyEd25519
	orig [PublicKeySize]byte
}

func boringPublicKey(pub PublicKey) (*boring.PublicKeyEd25519, error) {
	// Use the pointer to the underlying pub array as key.
	p := unsafe.SliceData(pub)
	b := pubCache.Get(p)
	if b != nil && PublicKey(b.orig[:]).Equal(pub) {
		return b.key, nil
	}

	b = new(boringPub)
	copy(b.orig[:], pub)
	key, err := boring.NewPublicKeyEd25119(b.orig[:])
	if err != nil {
		return nil, err
	}
	b.key = key
	pubCache.Put(p, b)
	return key, nil
}

type boringPriv struct {
	key  *boring.PrivateKeyEd25519
	orig [PrivateKeySize]byte
}

func boringPrivateKey(priv PrivateKey) (*boring.PrivateKeyEd25519, error) {
	// Use the pointer to the underlying priv array as key.
	p := unsafe.SliceData(priv)
	b := privCache.Get(p)
	if b != nil && PrivateKey(b.orig[:]).Equal(priv) {
		return b.key, nil
	}

	b = new(boringPriv)
	copy(b.orig[:], priv)
	key, err := boring.NewPrivateKeyEd25119(b.orig[:])
	if err != nil {
		return nil, err
	}
	b.key = key
	privCache.Put(p, b)
	return key, nil
}
